/***************************************************************************
 *   Copyright (C) 2009 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 **************************************************************************/

#include "KchildlockDaemon.h"
#include <KAboutData>
#include <KPluginFactory>
#include <klocalizedstring.h>
#include <KComponentData>
#include "../common/kchildlockcommon.h"
#include "kchildlock_general.h"
#include "systray_status.h"


K_PLUGIN_FACTORY(KchildlockFactory,
                 registerPlugin<KchildlockDaemon>();)
K_EXPORT_PLUGIN(KchildlockFactory("kchildlockdaemon"))


// the main program 
//=================

KchildlockDaemon::KchildlockDaemon(QObject *parent, const QList<QVariant>&)
        : KDEDModule(parent)
{
  
    QDate currdate;
    QTime currtime;
  
    KAboutData aboutData("kchildlockdaemon", "kchildlockdaemon", ki18n("Kchildlock Daemon"),
                         KCHILDLOCK_VERSION, ki18n("A Daemon for Kchildlock for KDE4"),
                         KAboutData::License_GPL, ki18n("(c), 2010 Rene Landert"),
                         KLocalizedString(), "This daemon "
                             "restricts the usage time of the computer per day and "
                             "per week for selected users.");

    aboutData.addAuthor(ki18n("Rene Landert"), ki18n("Maintainer"),  "rene.landert@bluewin.ch", "http://www.sourceforge.net/kchildlock");
    KGlobal::locale()->insertCatalog("kchildlock");
    
    KComponentData applicationData;
    applicationData = KComponentData(aboutData);
 

//    KComponentData *applicationData = new(KComponentData(aboutData));

    notificationTimer = new QTimer(this);
    startupTimer = new QTimer(this);
    scanTimer = new QTimer(this);

    QString enable_debuglog_filename = debuglog_filename_part1;
    enable_debuglog_filename.append(debuglog_filename_part3);
    QFile enable_debuglog_file ( enable_debuglog_filename );
    if ( enable_debuglog_file.exists() ) {
       debugflag = true;
       QString debuglog_filename = debuglog_filename_part1;
       QDateTime currdt = QDateTime::currentDateTime ();
       debuglog_filename.append(currdt.toString ("yyyyMMdd_hhmmss"));
       debuglog_filename.append(debuglog_filename_part3);
       debugfile = fopen ( debuglog_filename.toAscii(), "a+");
       }
    else
       debugflag = false;
    
    if (debugflag) {
       fprintf (debugfile,"\nStarting kchildlock daemon...\n");
       fprintf (debugfile,"date and time is %i.%i.%i. %i:%i:%i\n",
	QDate::currentDate().day(), QDate::currentDate().month(), QDate::currentDate().year(),
	QTime::currentTime().hour(), QTime::currentTime().minute(), QTime::currentTime().second());
       fflush(debugfile);
       }


    connect(startupTimer, SIGNAL(timeout()), this, SLOT(startupTimePassed()));
    startupTimer->start(5 * 1000);

}
    

// Destroy the object
// ==================

KchildlockDaemon::~KchildlockDaemon()
{
    delete ckSessionInterface;
}




// after the initial pause begin work
// ==================================
void KchildlockDaemon::startupTimePassed()
{

   bool retvalue;
   
//   data->my_actuserstate.date = QDate::currentDate();
//   data->my_actuserstate.time = QTime::currentTime();

   if (debugflag) {
      fprintf (debugfile,"\nafter the initial timeout...\n");
      fprintf (debugfile,"date and time is %i.%i.%i. %i:%i:%i\n",
	QDate::currentDate().day(), QDate::currentDate().month(), QDate::currentDate().year(),
	QTime::currentTime().hour(), QTime::currentTime().minute(), QTime::currentTime().second());
      fflush (debugfile);
      }

   startupTimer->stop();

//  First things first: let's set up Kchildlock to be aware of active session
    current_user = new CurrentUserState();
    QDBusConnection conn = QDBusConnection::systemBus();
    setUpConsoleKit();

   get_active_user ();
   current_user->set_warn_usr_state(KCHILDLOCK_NOWARNSTATE);
   current_user->set_warn_app_state(KCHILDLOCK_NOWARNSTATE);
   my_consumption = new Consumption();
   retvalue = my_consumption->open(current_user->get_user());

   if (debugflag) {
      fprintf (debugfile,"filename %s\n", my_consumption->get_consumption_filename().toAscii().constData());
      fflush(debugfile);
      }

// get values from consumed time file
    if ( retvalue )         // file could be opened
    {
            my_consumption->set_default_values(current_user->get_user());
            my_consumption->read();

            if (debugflag) {
               fprintf (debugfile,"consumed times values from file;\n");
               fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->get_weekofyear(), my_consumption->get_dayofweek(), my_consumption->pcusage_daily, my_consumption->pcusage_weekly);
               fprintf (debugfile,"app1=%s, usd=%i, usw=%i\n", my_consumption->appname1.toAscii().constData(), my_consumption->app1usage_daily, my_consumption->app1usage_weekly);
               fprintf (debugfile,"app2=%s, usd=%i, usw=%i\n", my_consumption->appname2.toAscii().constData(), my_consumption->app2usage_daily, my_consumption->app2usage_weekly);
               fprintf (debugfile,"app3=%s, usd=%i, usw=%i\n", my_consumption->appname3.toAscii().constData(), my_consumption->app3usage_daily, my_consumption->app3usage_weekly);
               fflush(debugfile);
            }
            
            my_consumption->close();
      }
      else
// set default values
      {
         my_consumption->set_default_values(current_user->get_user());
         if (debugflag) {
            fprintf (debugfile,"default consumed times values without file;\n");
            fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->get_weekofyear(), my_consumption->get_dayofweek(), my_consumption->pcusage_daily, my_consumption->pcusage_weekly);
            fflush(debugfile);
          }
      }

   if ( my_consumption->get_weekofyear() != QDate::currentDate().weekNumber() )
      {
      // its a new week, reset all consumed times values to zero
      my_consumption->set_default_values(current_user->get_user());
      if (debugflag) {
         fprintf (debugfile,"default consumed times values for a new week;\n");
         fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->get_weekofyear(), my_consumption->get_dayofweek(), my_consumption->pcusage_daily, my_consumption->pcusage_weekly);
         fflush(debugfile);
         }
      }


//    myGeneralSettings = new KchildlockGeneralSettings(); 
    myGeneralSettings = KSharedConfig::openConfig("kchildlockrc")->group("General");
    myUser1Settings = new KchildlockUserSettings(1);
    myUser2Settings = new KchildlockUserSettings(2);
    myUser3Settings = new KchildlockUserSettings(3);
    myUser4Settings = new KchildlockUserSettings(4);
    myUser5Settings = new KchildlockUserSettings(5);
    myGroup1Settings = new KchildlockGroupSettings(1);
    myGroup2Settings = new KchildlockGroupSettings(2);
    myGroup3Settings = new KchildlockGroupSettings(3);

// get global config settings
//    myGeneralSettings->readConfig();
//    scanint =  myGeneralSettings->scaninterval();
    scanint =  myGeneralSettings.readEntry("scaninterval").toInt();
//    if (myGeneralSettings->enable_restrictions()) {
    if (myGeneralSettings.readEntry("enable_restrictions").toInt()) {
       return;		// do nothing if disabled
    }
    
// create limits object
   current_limits = new KchildlockLimits();
   monitor_user();

    if (!myGeneralSettings.readEntry("enable_systray").toInt()) {
      if (!current_limits->get_nolimits()) {
         MainWindow *window = new MainWindow(0, current_limits, my_consumption);
      }
    }
   
  connect(scanTimer, SIGNAL(timeout()), this, SLOT(scanTimePassed()));
  scanTimer->start(1000 * scanint);

}




// after each scanintervall, moitor the system
// ===========================================
void KchildlockDaemon::scanTimePassed()
{
//data->my_actuserstate.date = QDate::currentDate();
//data->my_actuserstate.time = QTime::currentTime();
   scanTimer->stop();
   scanTimer->start(1000 * scanint);
   if (debugflag) {
      fprintf (debugfile,"\nafter the repeating timeout...\n");
      fprintf (debugfile,"date and time is %i.%i.%i. %i:%i:%i\n",
	QDate::currentDate().day(), QDate::currentDate().month(), QDate::currentDate().year(),
	QTime::currentTime().hour(), QTime::currentTime().minute(), QTime::currentTime().second());
      fprintf (debugfile,"actual user is %s\n", current_user->get_user().toAscii().constData());
      fflush(debugfile);
      }

   get_active_user ();
   monitor_user();

}




// the active session has changed, get new active User
// ===================================================
void KchildlockDaemon::activeSessionChanged()
{
// get the active user
//   data->my_actuserstate.date = QDate::currentDate();
//   data->my_actuserstate.time = QTime::currentTime();
   get_active_user();
   current_user->set_warn_usr_state(KCHILDLOCK_NOWARNSTATE);
   current_user->set_warn_app_state(KCHILDLOCK_NOWARNSTATE);

}




// setup the Communication to the console Kit to be informed about active session
// ==============================================================================
void KchildlockDaemon::setUpConsoleKit()
{
    // Let's cache the needed information to check if our session is actually active

    if (!QDBusConnection::systemBus().interface()->isServiceRegistered("org.freedesktop.ConsoleKit")) {
        // No way to determine if we are on the current session, simply suppose we are
        kDebug() << "Can't contact ck";
        ckAvailable = false;
        return;
    } else {
        ckAvailable = true;
    }

    // Otherwise, let's ask ConsoleKit
    QDBusInterface ckiface("org.freedesktop.ConsoleKit", "/org/freedesktop/ConsoleKit/Manager",
                           "org.freedesktop.ConsoleKit.Manager", QDBusConnection::systemBus());

    QDBusReply<QDBusObjectPath> sessionPath = ckiface.call("GetCurrentSession");

    if (!sessionPath.isValid() || sessionPath.value().path().isEmpty()) {
        kDebug() << "The session is not registered with ck";
        ckAvailable = false;
        return;
    }

    ckSessionInterface = new QDBusInterface("org.freedesktop.ConsoleKit", sessionPath.value().path(),
            "org.freedesktop.ConsoleKit.Session", QDBusConnection::systemBus());

    if (!ckSessionInterface->isValid()) {
        // As above
        kDebug() << "Can't contact Session Interface";
        ckAvailable = false;
        return;
    }

    /* If everything's fine, let's attach ourself to the ActiveChanged signal.
     * You'll see we're attaching to activeSessionChanged without any further checks.
     * You know why? activeSessionChanged already checks if the console is active, 
     * so the check is already happening n the underhood
     */

    QDBusConnection::systemBus().connect("org.freedesktop.ConsoleKit", sessionPath.value().path(),                                     "org.freedesktop.ConsoleKit.Session", "ActiveChanged", this, SLOT(activeSessionChanged()));

// get the active user
    get_active_user();

}


// functions for the handling of the current user
// ==============================================

CurrentUserState::CurrentUserState()
{
}

CurrentUserState::~CurrentUserState()
{
}

QString CurrentUserState::get_user()
{
   return user;
}

QString CurrentUserState::get_disp()
{
   return disp;
}

void CurrentUserState::set_user(QString newuser)
{
   user = newuser;
}

void CurrentUserState::set_disp(QString newdisp)
{
   disp = newdisp;
}

void CurrentUserState::set_app1isactive(int value)
{
   app1isactive = value;
}

void CurrentUserState::set_app2isactive(int value)
{
   app2isactive = value;
}

void CurrentUserState::set_app3isactive(int value)
{
   app3isactive = value;
}

int CurrentUserState::get_app1isactive()
{
   return(app1isactive);
}

int CurrentUserState::get_app2isactive()
{
   return(app2isactive);
}

int CurrentUserState::get_app3isactive()
{
   return(app3isactive);
}


#include "moc_KchildlockDaemon.cpp"
