/***************************************************************************
 *   Copyright (C) 2009 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KCHILDLOCKDAEMON_H
#define KCHILDLOCKDAEMON_H

#define KCHILDLOCK_WARNTIME3 900
#define KCHILDLOCK_WARNTIME2 600
#define KCHILDLOCK_WARNTIME1 300

#define KCHILDLOCK_NOWARNSTATE 10
#define KCHILDLOCK_WARNSTATE3 3
#define KCHILDLOCK_WARNSTATE2 2
#define KCHILDLOCK_WARNSTATE1 1
#define KCHILDLOCK_WARNSTATE0 0
#define KCHILDLOCK_WARNSTATE00 -1


#define debuglog_filename_part1 "/var/opt/kchildlock/kchildlockd_"
#define debuglog_filename_part3 "_debuglog.tmp"

#include <kdedmodule.h>
#include <KComponentData>
#include <QStringList>
#include "ksmserver_interface.h"
#include <KNotification>
#include "consumption_file.h"
#include "kchildlock_general.h"
#include "kchildlock_user.h"
#include "kchildlock_group.h"




class CurrentUserState
{
  
public:
   CurrentUserState();
   ~CurrentUserState();
   
private:
   QString user;
   QString disp;
   int warn_usr_state;
   int warn_app_state;
   int app1isactive, app2isactive, app3isactive;
   
public:
    QString get_user();
    QString get_disp();
    void set_user(QString newuser);
    void set_disp(QString newdisp);
    void set_warn_usr_state(int);
    void set_warn_app_state(int);
    int get_warn_usr_state();
    int get_warn_app_state();
    void set_app1isactive (int value);
    void set_app2isactive (int value);
    void set_app3isactive (int value);
    int get_app1isactive ();
    int get_app2isactive ();
    int get_app3isactive ();
};   


class KchildlockLimits
{

public:
    KchildlockLimits();
    ~KchildlockLimits();
    
    
private:
   bool no_limits;
   int daily, weekly, from, to;
   QString appname1, appname2, appname3;
   int app1_daily, app1_weekly, app1_from, app1_to;
   int app2_daily, app2_weekly, app2_from, app2_to;
   int app3_daily, app3_weekly, app3_from, app3_to;

public:
    void get_current_userlimits(QString user, KchildlockUserSettings *my_UserSettings, Consumption *my_consumption);
    void get_current_grouplimits(QString user, KchildlockGroupSettings *my_GroupSettings);
    void set_nolimits(bool value);
    bool get_nolimits();
    int get_lim_du();
    int get_lim_wu();
    int get_lim_from();
    int get_lim_to();
    QString get_appname1();
    QString get_appname2();
    QString get_appname3();
    

};



class KDE_EXPORT KchildlockDaemon : public KDEDModule
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.Kchildlock")

public:
    KchildlockDaemon(QObject *parent, const QList<QVariant>&);
    virtual ~KchildlockDaemon();
    
    
private:
   CurrentUserState *current_user;
   Consumption *my_consumption;
   KchildlockLimits *current_limits;
//   KSystemTrayIcon *my_systray;

    OrgKdeKSMServerInterfaceInterface *ksmServerIface;
    QDBusInterface *ckSessionInterface;
    bool ckAvailable;

//    KchildlockGeneralSettings *myGeneralSettings;    
    KConfigGroup myGeneralSettings;
    KchildlockUserSettings *myUser1Settings;    
    KchildlockUserSettings *myUser2Settings;    
    KchildlockUserSettings *myUser3Settings;    
    KchildlockUserSettings *myUser4Settings;    
    KchildlockUserSettings *myUser5Settings;    
    KchildlockGroupSettings *myGroup1Settings;    
    KchildlockGroupSettings *myGroup2Settings;    
    KchildlockGroupSettings *myGroup3Settings;    
    
    
    QTimer *notificationTimer;
    QTimer *startupTimer;
    QTimer *scanTimer;

    int scanint;	// if zero, then monitoring is disabled
    bool debugflag;
    FILE *debugfile;

public Q_SLOTS:
    void activeSessionChanged();

private Q_SLOTS:
    void startupTimePassed();
    void scanTimePassed();

Q_SIGNALS:
    
private:
    void get_active_user();
    void setUpConsoleKit();
    void monitor_user();
    void check_user_against_limits(KchildlockLimits *current_limits, QString actuser);
    void check_daily_usage_limits();
    void check_usage_limit_dwu(int secondofday, int limit_to);
    void warning_or_logout_session(int level);
    void force_logout_session();
    void force_app_termination(QString appname);
    void warning_or_finish_application (QString appname, int warn_level);
//    void create_systrayicon();
    
private slots:
    
signals:
    void triggered(bool);
};


#endif /*KCHILDLOCKDAEMON_H*/
