/***************************************************************************
 *   Copyright (C) 2009 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 **************************************************************************/

#include "../common/kchildlockcommon.h"
#include "consumption_file.h"
#include <QDate>
#include <QDataStream>


Consumption::Consumption()
{
  Consumption::set_default_values ("");
};


Consumption::~Consumption()
{
};

// open file with the consumed times
// =================================
bool Consumption::open(QString user)
{
   username = user;

   char outbfr[50], inbfr[50];
   const char *inptr = inbfr;
   inptr = username.toAscii().constData();
   uint i;

   for (i=0; i<sizeof(inptr); i++) {
      outbfr[2*i] = 'A' + (inptr[i]) / 16;
      outbfr[2*i+1] = 'A' + (inptr[i]) % 16;
      };
   outbfr[2*i] = 0;

   consumption_filename = consumption_filename_part1;
   consumption_filename.append(QString(outbfr));
   consumption_filename.append(consumption_filename_part3);

   consumption_file = new QFile( consumption_filename );
   if (consumption_file->exists ( consumption_filename ) ) {
      if ( consumption_file->open ( QIODevice::ReadWrite ) ) {
          return(TRUE);
      };
   }
   else {
   consumption_file->open ( QIODevice::ReadWrite );
   };
   return(FALSE);

};



// set the default valuse for the consumed times
// =============================================
void Consumption::set_default_values (QString user)
{

   version = KCHILDLOCK_VERSION;
   weekofyear = QDate::currentDate().weekNumber();
   dayofweek = QDate::currentDate().dayOfWeek();
   username = user;
   pcusage_daily = 0;
   pcusage_weekly = 0;

   app1usage_daily = 0;
   app2usage_daily = 0;
   app3usage_daily = 0;
   app1usage_weekly = 0;
   app2usage_weekly = 0;
   app3usage_weekly = 0;

};


// reset the daily valuse for a new day
// ====================================
void Consumption::reset_daily_values ()
{

   weekofyear = QDate::currentDate().weekNumber();
   dayofweek = QDate::currentDate().dayOfWeek();
   pcusage_daily = 0;
   app1usage_daily = 0;
   app2usage_daily = 0;
   app3usage_daily = 0;

};


// read the values from the consumed time file
// ===========================================
void Consumption::read()
{

   QDataStream stream ( consumption_file ); // read the data serialized from the file
   stream >> version;
   stream >> weekofyear;
   stream >> dayofweek;
   stream >> username;
   stream >> pcusage_daily;
   stream >> pcusage_weekly;
   stream >> appname1;
   stream >> app1usage_daily;
   stream >> app1usage_weekly;
   stream >> appname2;
   stream >> app2usage_daily;
   stream >> app2usage_weekly;
   stream >> appname3;
   stream >> app3usage_daily;
   stream >> app3usage_weekly;

};


// write the values to consumed time file
// ======================================
void Consumption::write()
{

   QDataStream stream ( consumption_file ); // read the data serialized from the file
   stream << version;
   stream << weekofyear;
   stream << dayofweek;
   stream << username;
   stream << pcusage_daily;
   stream << pcusage_weekly;
   stream << appname1;
   stream << app1usage_daily;
   stream << app1usage_weekly;
   stream << appname2;
   stream << app2usage_daily;
   stream << app2usage_weekly;
   stream << appname3;
   stream << app3usage_daily;
   stream << app3usage_weekly;

};




// flush and close the working set file
// ====================================
void Consumption::close()
{
    consumption_file->flush();
    consumption_file->close();
};


// return the appropriate values
//==============================
QString Consumption::get_consumption_filename()
{
   return consumption_filename;
};



int Consumption::get_weekofyear()
{
   return weekofyear;
};

int Consumption::get_dayofweek()
{
   return dayofweek;
};

