/***************************************************************************
 *   Copyright (C) 2009 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "KchildlockDaemon.h"
#include "kchildlock_user.h"
#include "kchildlock_group.h"

#include <kuser.h>
#include <qdatetime.h>


// Constructor of class
// ====================
KchildlockLimits::KchildlockLimits()
{
  daily=24*60*60;
  weekly=7*24*60*60;
  from=0;
  to=24*60*60,
  appname1="";
  appname2="";
  appname3="";
  no_limits=TRUE;
}

// Destructor of class
// ====================
KchildlockLimits::~KchildlockLimits()
{
}

// set the no limitations flag
// ===========================
void KchildlockLimits::set_nolimits(bool value)
{
  no_limits = value;
}

// get the no limitations flag
// ===========================
bool KchildlockLimits::get_nolimits()
{
  return(no_limits);
}

// get the limit values
// ====================
int KchildlockLimits::get_lim_du()
{
  return (daily);
}

int KchildlockLimits::get_lim_wu()
{
  return (weekly);
}
int KchildlockLimits::get_lim_from()
{
  return (from);
}
int KchildlockLimits::get_lim_to()
{
  return (to);
}
QString KchildlockLimits::get_appname1()
{
  // appname1 = empty string means "not enabled2 too
  return (appname1);
}
QString KchildlockLimits::get_appname2()
{
  return (appname2);
}
QString KchildlockLimits::get_appname3()
{
  return (appname3);
}




// get the limitations for the user if its the active user
// =======================================================
void KchildlockLimits::get_current_userlimits(QString actuser, KchildlockUserSettings *act_user_settings, Consumption *my_consumption)

{

   if ((actuser != act_user_settings->usernamestring()) || !(act_user_settings->enabled()))
	return;

   QDate actdate = QDate::currentDate();
   if (act_user_settings->wEEKenabled()) {
      weekly = act_user_settings->wEEKmaxhr() * 3600 + act_user_settings->wEEKmaxmin() * 60;
   }

   if (act_user_settings->sameRestrict()) {
		daily = act_user_settings->dAYmaxhr() * 3600 + act_user_settings->dAYmaxmin() * 60;
		from = act_user_settings->dAYfrom() * 60;
		to = act_user_settings->dAYto() * 60;
		no_limits = FALSE;
   } else {

	if (actdate.dayOfWeek() == 1 && act_user_settings->mONenabled() )
	{
		daily = act_user_settings->mONmaxhr() * 3600 + act_user_settings->mONmaxmin() * 60;
		from = act_user_settings->mONfrom() * 60;
		to = act_user_settings->mONto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 2 && act_user_settings->tUEenabled())
	{
		daily = act_user_settings->tUEmaxhr() * 3600 + act_user_settings->tUEmaxmin() * 60;
		from = act_user_settings->tUEfrom() * 60;
		to = act_user_settings->tUEto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 3 && act_user_settings->wEDenabled())
	{
		daily = act_user_settings->wEDmaxhr() * 3600 + act_user_settings->wEDmaxmin() * 60;
		from = act_user_settings->wEDfrom() * 60;
		to = act_user_settings->wEDto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 4 && act_user_settings->tHUenabled())
	{
		daily = act_user_settings->tHUmaxhr() * 3600 + act_user_settings->tHUmaxmin() * 60;
		from = act_user_settings->tHUfrom() * 60;
		to = act_user_settings->tHUto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 5 && act_user_settings->fRIenabled())
	{
		daily = act_user_settings->fRImaxhr() * 3600 + act_user_settings->fRImaxmin() * 60;
		from = act_user_settings->fRIfrom() * 60;
		to = act_user_settings->fRIto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 6 && act_user_settings->sATenabled())
	{
		daily = act_user_settings->sATmaxhr() * 3600 + act_user_settings->sATmaxmin() * 60;
		from = act_user_settings->sATfrom() * 60;
		to = act_user_settings->sATto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 7 && act_user_settings->sUNenabled())
	{
		daily = act_user_settings->sUNmaxhr() * 3600 + act_user_settings->sUNmaxmin() * 60;
		from = act_user_settings->sUNfrom() * 60;
		to = act_user_settings->sUNto() * 60;
		no_limits = FALSE;
	}
	else
	{
		daily = 24 * 3600 + KCHILDLOCK_WARNTIME3;
		from = 0;
		to = 24 * 3600 + KCHILDLOCK_WARNTIME3;
		no_limits = FALSE;
	}
   }
   
   
   // get the limitations for the user applications
   // =============================================

   if (act_user_settings->aPP1enabled()) {
      appname1 = act_user_settings->appName1();
   } else {
      appname1 = "";
   }
   my_consumption->appname1 = act_user_settings->appName1();
   app1_daily = act_user_settings->app1maxDhr() * 3600 + act_user_settings->app1maxDmin() * 60;
   app1_weekly = act_user_settings->app1maxWhr() * 3600 + act_user_settings->app1maxWmin() * 60;
   app1_from = act_user_settings->app1from() * 60;
   app1_to = act_user_settings->app1to() * 60;

   if (act_user_settings->aPP2enabled()) {
      appname2 = act_user_settings->appName2();
   } else {
      appname2 = "";
   }
   my_consumption->appname2 = act_user_settings->appName2();
   app2_daily = act_user_settings->app2maxDhr() * 3600 + act_user_settings->app2maxDmin() * 60;
   app2_weekly = act_user_settings->app2maxWhr() * 3600 + act_user_settings->app2maxWmin() * 60;
   app2_from = act_user_settings->app2from() * 60;
   app2_to = act_user_settings->app2to() * 60;

   if (act_user_settings->aPP3enabled()) {
      appname3 = act_user_settings->appName3();
   } else {
      appname3 = "";
   }
   my_consumption->appname3 = act_user_settings->appName3();
   app3_daily = act_user_settings->app3maxDhr() * 3600 + act_user_settings->app3maxDmin() * 60;
   app3_weekly = act_user_settings->app3maxWhr() * 3600 + act_user_settings->app3maxWmin() * 60;
   app3_from = act_user_settings->app3from() * 60;
   app3_to = act_user_settings->app3to() * 60;

   
}









// get the limitations for group if its the active user
// ===================================================
void KchildlockLimits::get_current_grouplimits(QString actuser, KchildlockGroupSettings *act_group_settings)
{
   QStringList myuserlist;
   const char *mygroupn;
   KUserGroup mygroup;
   long mygid;
   bool ismember;

   mygroupn = (act_group_settings->groupnamestring()).toAscii().constData();
   mygroup = KUserGroup(mygroupn);
   myuserlist = mygroup.userNames();
   mygid = mygroup.gid();
   ismember = FALSE;

   for ( QStringList::Iterator it = myuserlist.begin(); it != myuserlist.end(); ++it ) {
	if ((*it) != actuser)
		ismember = TRUE;
   }
   if (!ismember || !(act_group_settings->enabled())) 
	return;


   QDate actdate = QDate::currentDate();
   if (act_group_settings->wEEKenabled()) {
      weekly = act_group_settings->wEEKmaxhr() * 3600 + act_group_settings->wEEKmaxmin() * 60;
   }

   if (act_group_settings->sameRestrict()) {
	daily = act_group_settings->dAYmaxhr() * 3600 + act_group_settings->dAYmaxmin() * 60;
	from = act_group_settings->dAYfrom() * 60;
	to = act_group_settings->dAYto() * 60;
	no_limits = FALSE;
   } else {
	if (actdate.dayOfWeek() == 1 && act_group_settings->mONenabled() )
	{
		daily = act_group_settings->mONmaxhr() * 3600 + act_group_settings->mONmaxmin() * 60;
		from = act_group_settings->mONfrom() * 60;
		to = act_group_settings->mONto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 2 && act_group_settings->tUEenabled())
	{
		daily = act_group_settings->tUEmaxhr() * 3600 + act_group_settings->tUEmaxmin() * 60;
		from = act_group_settings->tUEfrom() * 60;
		to = act_group_settings->tUEto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 3 && act_group_settings->wEDenabled())
	{
		daily = act_group_settings->wEDmaxhr() * 3600 + act_group_settings->wEDmaxmin() * 60;
		from = act_group_settings->wEDfrom() * 60;
		to = act_group_settings->wEDto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 4 && act_group_settings->tHUenabled())
	{
		daily = act_group_settings->tHUmaxhr() * 3600 + act_group_settings->tHUmaxmin() * 60;
		from = act_group_settings->tHUfrom() * 60;
		to = act_group_settings->tHUto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 5 && act_group_settings->fRIenabled())
	{
		daily = act_group_settings->fRImaxhr() * 3600 + act_group_settings->fRImaxmin() * 60;
		from = act_group_settings->fRIfrom() * 60;
		to = act_group_settings->fRIto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 6 && act_group_settings->sATenabled())
	{
		daily = act_group_settings->sATmaxhr() * 3600 + act_group_settings->sATmaxmin() * 60;
		from = act_group_settings->sATfrom() * 60;
		to = act_group_settings->sATto() * 60;
		no_limits = FALSE;
	}
	else if (actdate.dayOfWeek() == 7 && act_group_settings->sUNenabled())
	{
		daily = act_group_settings->sUNmaxhr() * 3600 + act_group_settings->sUNmaxmin() * 60;
		from = act_group_settings->sUNfrom() * 60;
		to = act_group_settings->sUNto() * 60;
		no_limits = FALSE;
	}
	else
	{
		daily = 24 * 3600 + KCHILDLOCK_WARNTIME3;
		from = 0;
		to = 24 * 3600 + KCHILDLOCK_WARNTIME3;
		no_limits = FALSE;
	}
   }
}




