 /***************************************************************************
 *   Copyright (C) 2009 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include <KPluginFactory>
#include <KAboutData>
#include <QVBoxLayout>
#include <QFile>
#include <KConfigDialog>
#include <KPageDialog>
#include <KConfigGroup>
#include <KUserGroup>
#include <KUser>
#include <kdebug.h>

#include "KchildlockKCM.h"
#include "kchildlock_general.h"
#include "kchildlock_user.h"
#include "kchildlock_group.h"
#include "ui_generalPage.h"
#include "ui_userPage.h"
#include "ui_groupPage.h"


K_PLUGIN_FACTORY(KchildlockFactory,
                 registerPlugin<KchildlockKCM>();
                )
K_EXPORT_PLUGIN(KchildlockFactory("kcm_kchildlock"))

KchildlockKCM::KchildlockKCM(QWidget *parent, const QVariantList &args) :
        KCModule(KchildlockFactory::componentData(), parent, args)
        
{
    KGlobal::locale()->insertCatalog("kchildlock");

    KAboutData *about =
        new KAboutData("kcmkchildlock", "kchildlock", ki18n("Kchildlock Configuration"),
                       KCHILDLOCK_VERSION, ki18n("A configurator for Kchildlock"),
                       KAboutData::License_GPL, ki18n("(c), 2010 Rene Landert"),
                       ki18n("From this module, you can configure the Kchildlock Daemon, "
                             "which restricts the usage time of the computer and of applications per day and "
                             "per week for selected users. "));

    about->addAuthor(ki18n("Rene Landert"), ki18n("Main Developer") , "rene.landert@bluewin.ch", "http://www.sourceforge.net/kchildlock");

    setAboutData(about);
    setButtons(KCModule::Apply|KCModule::Help|KCModule::Default);

    setQuickHelp(i18n("<h1>Kchildlock configuration</h1> <p>This module lets you configure "
                      "Kchildlock. Kchildlock is a daemon (so it runs in background) that is started "
                      "upon KDE startup.</p> <p>Kchildlock has 3 levels of configuration: a general one, "
                      "with common settings, 5 possible users for whom you can configure the restrictions, "
                      "and 3 groups for whom you can configure the restrictions."
                      "Each tab for the user or group lets you edit the restriction details. "
                      "You do not have to restart Kchildlock, just click on the button "
                      "'Apply', and you are done.</p>"));

    if (getuid() != 0){
        KMessageBox::information(0,i18n("Only root can change the settings of this program!"));
        setButtons(KCModule::Help);
    }

//    if ( KConfigDialog::showDialog( "settings" ) )  {
//        return;
//    }


    myGeneralSettings = new KchildlockGeneralSettings(); 
    myUser1Settings = new KchildlockUserSettings(1);
    myUser2Settings = new KchildlockUserSettings(2);
    myUser3Settings = new KchildlockUserSettings(3);
    myUser4Settings = new KchildlockUserSettings(4);
    myUser5Settings = new KchildlockUserSettings(5);
    myGroup1Settings = new KchildlockGroupSettings(1);
    myGroup2Settings = new KchildlockGroupSettings(2);
    myGroup3Settings = new KchildlockGroupSettings(3);
    dialog = new KPageDialog (parent);

    dialog->setButtons(0);
    setButtons(KCModule::Apply|KCModule::Help|KCModule::Default);    
    dialog->setModal(false);
    
    QVBoxLayout *mylayout = new QVBoxLayout(this);
    mylayout->setSpacing(KDialog::spacingHint());
    mylayout->setMargin(0);
    mylayout->addWidget(dialog);

//  configuration dialog for general settings
    QWidget *generalSettingsDlg = new QWidget(this);    
    general = new Ui_generalPage();
    general->setupUi( generalSettingsDlg );
    generalPageItem = dialog->addPage ( generalSettingsDlg, i18n ( "General") );
    generalPageItem->setIcon(KIcon("configure"));

//  configuration dialog for user1 settings
    QWidget *user1SettingsDlg = new QWidget(this);    
    user1 = new Ui_userPage();
    user1->setupUi( user1SettingsDlg );
    KchildlockKCM::setupUi_part2(user1, myUser1Settings, QString("1"));
    user1PageItem = dialog->addPage ( user1SettingsDlg, i18n ( "User 1" ) );
    user1PageItem->setIcon(KIcon("list-add-user"));

//  configuration dialog for user2 settings
    QWidget *user2SettingsDlg = new QWidget(this);    
    user2 = new Ui_userPage();
    user2->setupUi( user2SettingsDlg );	
    KchildlockKCM::setupUi_part2(user2, myUser2Settings, QString("2"));
    user2PageItem = dialog->addPage ( user2SettingsDlg, i18n ( "User 2" ) );
    user2PageItem->setIcon(KIcon("list-add-user"));

//  configuration dialog for user3 settings
    QWidget *user3SettingsDlg = new QWidget(this);    
    user3 = new Ui_userPage();
    user3->setupUi( user3SettingsDlg );	
    KchildlockKCM::setupUi_part2(user3, myUser3Settings, QString("3"));
    user3PageItem = dialog->addPage ( user3SettingsDlg, i18n ( "User 3" ) );
    user3PageItem->setIcon(KIcon("list-add-user"));

//  configuration dialog for user4 settings
    QWidget *user4SettingsDlg = new QWidget(this);    
    user4 = new Ui_userPage();
    user4->setupUi( user4SettingsDlg );	
    KchildlockKCM::setupUi_part2(user4, myUser4Settings, QString("4"));
    user4PageItem = dialog->addPage ( user4SettingsDlg, i18n ( "User 4" ) );
    user4PageItem->setIcon(KIcon("list-add-user"));

//  configuration dialog for user5 settings
    QWidget *user5SettingsDlg = new QWidget(this);    
    user5 = new Ui_userPage();
    user5->setupUi( user5SettingsDlg );	
    KchildlockKCM::setupUi_part2(user5, myUser5Settings, QString("5"));
    user5PageItem = dialog->addPage ( user5SettingsDlg, i18n ( "User 5" ) );
    user5PageItem->setIcon(KIcon("list-add-user"));

//  configuration dialog for group1 settings
    QWidget *group1SettingsDlg = new QWidget(this);    
    group1 = new Ui_groupPage();
    group1->setupUi( group1SettingsDlg );	
    KchildlockKCM::setupUi_part3(group1, myGroup1Settings, QString("1"));
    group1PageItem = dialog->addPage ( group1SettingsDlg, i18n ( "Group 1" ) );
    group1PageItem->setIcon(KIcon("user-group-new"));

//  configuration dialog for group2 settings
    QWidget *group2SettingsDlg = new QWidget(this);    
    group2 = new Ui_groupPage();
    group2->setupUi( group2SettingsDlg );	
    KchildlockKCM::setupUi_part3(group2, myGroup2Settings, QString("2"));
    group2PageItem = dialog->addPage ( group2SettingsDlg, i18n ( "Group 2" ) );
    group2PageItem->setIcon(KIcon("user-group-new"));

//  configuration dialog for group3 settings
    QWidget *group3SettingsDlg = new QWidget(this);    
    group3 = new Ui_groupPage();
    group3->setupUi( group3SettingsDlg );	
    KchildlockKCM::setupUi_part3(group3, myGroup3Settings, QString("3"));
    group3PageItem = dialog->addPage ( group3SettingsDlg, i18n ( "Group 3" ) );
    group3PageItem->setIcon(KIcon("user-group-new"));

    
//  connect signals and slots
    KchildlockKCM::connect_general(general);
    KchildlockKCM::connect_user(user1);
    KchildlockKCM::connect_user(user2);
    KchildlockKCM::connect_user(user3);
    KchildlockKCM::connect_user(user4);
    KchildlockKCM::connect_user(user5);
    KchildlockKCM::connect_group(group1);
    KchildlockKCM::connect_group(group2);
    KchildlockKCM::connect_group(group3);

//    KchildlockKCM::load();
    dialog->show();
    
}


KchildlockKCM::~KchildlockKCM()
{
}

void KchildlockKCM::load()
{
//  if (getuid() != 0) {
//        return;
//   }
   kDebug() << "KCM is loading config";
   myGeneralSettings->readConfig();
   myUser1Settings->readConfig();
   myUser2Settings->readConfig();
   myUser3Settings->readConfig();
   myUser4Settings->readConfig();
   myUser5Settings->readConfig();
   myGroup1Settings->readConfig();
   myGroup2Settings->readConfig();
   myGroup3Settings->readConfig();

   KchildlockKCM::fillValuesInGui_general();
   KchildlockKCM::fillValuesInGui_user(user1, myUser1Settings);
   KchildlockKCM::fillValuesInGui_user(user2, myUser2Settings);
   KchildlockKCM::fillValuesInGui_user(user3, myUser3Settings);
   KchildlockKCM::fillValuesInGui_user(user4, myUser4Settings);
   KchildlockKCM::fillValuesInGui_user(user5, myUser5Settings);
   KchildlockKCM::fillValuesInGui_group(group1, myGroup1Settings);
   KchildlockKCM::fillValuesInGui_group(group2, myGroup2Settings);
   KchildlockKCM::fillValuesInGui_group(group3, myGroup3Settings);
   
   if (getuid() !=0) {
     dialog->enableButtonApply(FALSE);
     dialog->enableButtonOk(FALSE);
// future extension: disable all widgets
    }

    connect(this, SIGNAL(updateMe()), user1->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_App1graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_App2graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user1->kcfg_App3graphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), user2->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_App1graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_App2graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user2->kcfg_App3graphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), user3->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_App1graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_App2graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user3->kcfg_App3graphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), user4->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_App1graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_App2graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user4->kcfg_App3graphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), user5->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_App1graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_App2graphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), user5->kcfg_App3graphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), group1->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group1->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), group2->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group2->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));

    connect(this, SIGNAL(updateMe()), group3->kcfg_DAYgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_MONgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_TUEgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_WEDgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_THUgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_FRIgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_SATgraphics->scene(), SLOT(slotUpdateMe()));
    connect(this, SIGNAL(updateMe()), group3->kcfg_SUNgraphics->scene(), SLOT(slotUpdateMe()));

    emit updateMe();
   
}

void KchildlockKCM::save()
{
  if (getuid() != 0) {
        return;
   }
//   kDebug() << "KCM is saving config";
   myGeneralSettings->writeConfig();
   myUser1Settings->writeConfig();
   myUser2Settings->writeConfig();
   myUser3Settings->writeConfig();
   myUser4Settings->writeConfig();
   myUser5Settings->writeConfig();
   myGroup1Settings->writeConfig();
   myGroup2Settings->writeConfig();
   myGroup3Settings->writeConfig();

// write same file from root's home to the global config file
   QString newconfigpath = KStandardDirs::installPath("config").append("kchildlockrc");
//   kDebug() << "newconfigpath is " << newconfigpath ;
   QFile::remove(newconfigpath);
   if (QFile::exists("/root/.kde4/share/config/kchildlockrc")) {
      QFile::copy("/root/.kde4/share/config/kchildlockrc", newconfigpath);
   } else {
      QFile::copy("/root/.kde/share/config/kchildlockrc", newconfigpath);
   };
   QFile::setPermissions(newconfigpath, QFile::ReadUser|QFile::WriteUser|QFile::ReadGroup|QFile::ReadOther);

}


Ui_userPage *KchildlockKCM::current_userPage()
{
   KPageWidgetItem *curr_page = dialog->currentPage();
   if ( curr_page == user1PageItem ) {
     mycurrent_UserSettings = myUser1Settings;
     return (user1);
   }
   if ( curr_page == user2PageItem ) {
     mycurrent_UserSettings = myUser2Settings;
     return (user2);
   }
   if ( curr_page == user3PageItem ) {
     mycurrent_UserSettings = myUser3Settings;
     return (user3);
   }
   if ( curr_page == user4PageItem ) {
     mycurrent_UserSettings = myUser4Settings;
     return (user4);
   }
   if ( curr_page == user5PageItem ) {
     mycurrent_UserSettings = myUser5Settings;
     return (user5);
   }
   return (0);
}


Ui_groupPage *KchildlockKCM::current_groupPage()
{
   KPageWidgetItem *curr_page = dialog->currentPage();
   if ( curr_page == group1PageItem ) {
     mycurrent_GroupSettings = myGroup1Settings;
     return (group1);
   }
   if ( curr_page == group2PageItem ) {
     mycurrent_GroupSettings = myGroup2Settings;
     return (group2);
   }
   if ( curr_page == group3PageItem ) {
     mycurrent_GroupSettings = myGroup3Settings;
     return (group3);
   }
   return (0);
}


QStringList KchildlockKCM::getlistofusers()
{
    KUser* myKusers = new KUser();
    KUser myKuser;
    QStringList fulluserlist, effuserlist;
    fulluserlist = myKusers->allUserNames();
    effuserlist = QStringList();
    for ( QStringList::Iterator it = fulluserlist.begin(); it != fulluserlist.end(); ++it ) {
	myKuser = KUser(*it);
	if (myKuser.uid() >= USERIDLOWLIMIT) {
	   effuserlist.append(*it);
	}
    }		
    return effuserlist;
}


QStringList KchildlockKCM::getlistofgroups()
{
    KUserGroup* myKgroups = new KUserGroup();
    KUserGroup myKgroup;
    QStringList fullgrouplist, effgrouplist;
    fullgrouplist = myKgroups->allGroupNames();
    effgrouplist = QStringList();
    for ( QStringList::Iterator it = fullgrouplist.begin(); it != fullgrouplist.end(); ++it ) {
	myKgroup = KUserGroup(*it);
	if (myKgroup.gid() >= GROUPIDLOWLIMIT) {
	   effgrouplist.append(*it);
	}
    }		
    return effgrouplist;
}


QString KchildlockKCM::quickHelp() const
{
  return i18n("<h1>Kchildlock configuration</h1><p>This module allows you to configure your childrens restriction settings.</p>");
}


void  KchildlockKCM::setupUi_part2(Ui_userPage *user, KchildlockUserSettings *mySettings, QString numberstring)
{
    user->kcfg_username->insertItem(-1, mySettings->usernamestring());
    user->kcfg_username->insertItems (1, this->getlistofusers());
    user->labelA->setText(user->labelA->text() + " " + numberstring);
    user->kcfg_DAYruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_DAYruler->setMediumMarkDistance(8);
    user->kcfg_DAYruler->setOffset(0);
    my_scene = new MyScene(user->widget_DAY, user->kcfg_DAYgraphics->geometry(), user->kcfg_DAYfrom, user->kcfg_DAYto );
    user->kcfg_DAYgraphics->setScene(my_scene);
    user->kcfg_DAYgraphics->setMouseTracking(TRUE);
    user->kcfg_DAYgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_DAYgraphics->setInteractive(TRUE);
    user->kcfg_DAYgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_DAYgraphics->show();
    user->kcfg_DAYfrom->hide();
    user->kcfg_DAYto->hide();

    user->kcfg_MONruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_MONruler->setMediumMarkDistance(8);
    user->kcfg_MONruler->setOffset(0);
    my_scene = new MyScene(user->widget_MON, user->kcfg_MONgraphics->geometry(), user->kcfg_MONfrom, user->kcfg_MONto );
    user->kcfg_MONgraphics->setScene(my_scene);
    user->kcfg_MONgraphics->setMouseTracking(TRUE);
    user->kcfg_MONgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_MONgraphics->setInteractive(TRUE);
    user->kcfg_MONgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_MONgraphics->show();
    user->kcfg_MONfrom->hide();
    user->kcfg_MONto->hide();

    user->kcfg_TUEruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_TUEruler->setMediumMarkDistance(8);
    user->kcfg_TUEruler->setOffset(0);
    my_scene = new MyScene(user->widget_TUE, user->kcfg_TUEgraphics->geometry(), user->kcfg_TUEfrom, user->kcfg_TUEto );
    user->kcfg_TUEgraphics->setScene(my_scene);
    user->kcfg_TUEgraphics->setMouseTracking(TRUE);
    user->kcfg_TUEgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_TUEgraphics->setInteractive(TRUE);
    user->kcfg_TUEgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_TUEgraphics->show();
    user->kcfg_TUEfrom->hide();
    user->kcfg_TUEto->hide();

    user->kcfg_WEDruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_WEDruler->setMediumMarkDistance(8);
    user->kcfg_WEDruler->setOffset(0);
    my_scene = new MyScene(user->widget_WED, user->kcfg_WEDgraphics->geometry(), user->kcfg_WEDfrom, user->kcfg_WEDto );
    user->kcfg_WEDgraphics->setScene(my_scene);
    user->kcfg_WEDgraphics->setMouseTracking(TRUE);
    user->kcfg_WEDgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_WEDgraphics->setInteractive(TRUE);
    user->kcfg_WEDgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_WEDgraphics->show();
    user->kcfg_WEDfrom->hide();
    user->kcfg_WEDto->hide();

    user->kcfg_THUruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_THUruler->setMediumMarkDistance(8);
    user->kcfg_THUruler->setOffset(0);
    my_scene = new MyScene(user->widget_THU, user->kcfg_THUgraphics->geometry(), user->kcfg_THUfrom, user->kcfg_THUto );
    user->kcfg_THUgraphics->setScene(my_scene);
    user->kcfg_THUgraphics->setMouseTracking(TRUE);
    user->kcfg_THUgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_THUgraphics->setInteractive(TRUE);
    user->kcfg_THUgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_THUgraphics->show();
    user->kcfg_THUfrom->hide();
    user->kcfg_THUto->hide();

    user->kcfg_FRIruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_FRIruler->setMediumMarkDistance(8);
    user->kcfg_FRIruler->setOffset(0);
    my_scene = new MyScene(user->widget_FRI, user->kcfg_FRIgraphics->geometry(), user->kcfg_FRIfrom, user->kcfg_FRIto );
    user->kcfg_FRIgraphics->setScene(my_scene);
    user->kcfg_FRIgraphics->setMouseTracking(TRUE);
    user->kcfg_FRIgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_FRIgraphics->setInteractive(TRUE);
    user->kcfg_FRIgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_FRIgraphics->show();
    user->kcfg_FRIfrom->hide();
    user->kcfg_FRIto->hide();

    user->kcfg_SATruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_SATruler->setMediumMarkDistance(8);
    user->kcfg_SATruler->setOffset(0);
    my_scene = new MyScene(user->widget_SAT, user->kcfg_SATgraphics->geometry(), user->kcfg_SATfrom, user->kcfg_SATto );
    user->kcfg_SATgraphics->setScene(my_scene);
    user->kcfg_SATgraphics->setMouseTracking(TRUE);
    user->kcfg_SATgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_SATgraphics->setInteractive(TRUE);
    user->kcfg_SATgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_SATgraphics->show();
    user->kcfg_SATfrom->hide();
    user->kcfg_SATto->hide();

    user->kcfg_SUNruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_SUNruler->setMediumMarkDistance(8);
    user->kcfg_SUNruler->setOffset(0);
    my_scene = new MyScene(user->widget_SUN, user->kcfg_SUNgraphics->geometry(), user->kcfg_SUNfrom, user->kcfg_SUNto );
    user->kcfg_SUNgraphics->setScene(my_scene);
    user->kcfg_SUNgraphics->setMouseTracking(TRUE);
    user->kcfg_SUNgraphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_SUNgraphics->setInteractive(TRUE);
    user->kcfg_SUNgraphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_SUNgraphics->show();
    user->kcfg_SUNfrom->hide();
    user->kcfg_SUNto->hide();

    user->kcfg_App1ruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_App1ruler->setMediumMarkDistance(8);
    user->kcfg_App1ruler->setOffset(0);
    my_scene = new MyScene(user->widget_APP1, user->kcfg_SUNgraphics->geometry(), user->kcfg_SUNfrom, user->kcfg_SUNto );
    user->kcfg_App1graphics->setScene(my_scene);
    user->kcfg_App1graphics->setMouseTracking(TRUE);
    user->kcfg_App1graphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_App1graphics->setInteractive(TRUE);
    user->kcfg_App1graphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_App1graphics->show();
    user->kcfg_App1from->hide();
    user->kcfg_App1to->hide();

    user->kcfg_App2ruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_App2ruler->setMediumMarkDistance(8);
    user->kcfg_App2ruler->setOffset(0);
    my_scene = new MyScene(user->widget_APP2, user->kcfg_SUNgraphics->geometry(), user->kcfg_SUNfrom, user->kcfg_SUNto );
    user->kcfg_App2graphics->setScene(my_scene);
    user->kcfg_App2graphics->setMouseTracking(TRUE);
    user->kcfg_App2graphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_App2graphics->setInteractive(TRUE);
    user->kcfg_App2graphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_App2graphics->show();
    user->kcfg_App2from->hide();
    user->kcfg_App2to->hide();

    user->kcfg_App3ruler->setRulerMetricStyle(KRuler::Custom);
    user->kcfg_App3ruler->setMediumMarkDistance(8);
    user->kcfg_App3ruler->setOffset(0);
    my_scene = new MyScene(user->widget_APP3, user->kcfg_SUNgraphics->geometry(), user->kcfg_SUNfrom, user->kcfg_SUNto );
    user->kcfg_App3graphics->setScene(my_scene);
    user->kcfg_App3graphics->setMouseTracking(TRUE);
    user->kcfg_App3graphics->viewport()->setMouseTracking(TRUE);
    user->kcfg_App3graphics->setInteractive(TRUE);
    user->kcfg_App3graphics->setFocusPolicy(Qt::StrongFocus);
    user->kcfg_App3graphics->show();
    user->kcfg_App3from->hide();
    user->kcfg_App3to->hide();

}


void  KchildlockKCM::setupUi_part3(Ui_groupPage *group, KchildlockGroupSettings *mySettings, QString numberstring)
{
    group->kcfg_groupname->insertItem(-1, mySettings->groupnamestring());
    group->kcfg_groupname->insertItems (1, this->getlistofgroups());
    group->labelA->setText(group->labelA->text() + " " + numberstring);
    group->kcfg_DAYruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_DAYruler->setMediumMarkDistance(8);
    group->kcfg_DAYruler->setOffset(0);
    my_scene = new MyScene(group->widget_DAY, group->kcfg_DAYgraphics->geometry(), group->kcfg_DAYfrom, group->kcfg_DAYto );
    group->kcfg_DAYgraphics->setScene(my_scene);
    group->kcfg_DAYgraphics->setMouseTracking(TRUE);
    group->kcfg_DAYgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_DAYgraphics->setInteractive(TRUE);
    group->kcfg_DAYgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_DAYgraphics->show();
    group->kcfg_DAYfrom->hide();
    group->kcfg_DAYto->hide();

    group->kcfg_MONruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_MONruler->setMediumMarkDistance(8);
    group->kcfg_MONruler->setOffset(0);
    my_scene = new MyScene(group->widget_MON, group->kcfg_MONgraphics->geometry(), group->kcfg_MONfrom, group->kcfg_MONto );
    group->kcfg_MONgraphics->setScene(my_scene);
    group->kcfg_MONgraphics->setMouseTracking(TRUE);
    group->kcfg_MONgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_MONgraphics->setInteractive(TRUE);
    group->kcfg_MONgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_MONgraphics->show();
    group->kcfg_MONfrom->hide();
    group->kcfg_MONto->hide();

    group->kcfg_TUEruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_TUEruler->setMediumMarkDistance(8);
    group->kcfg_TUEruler->setOffset(0);
    my_scene = new MyScene(group->widget_TUE, group->kcfg_TUEgraphics->geometry(), group->kcfg_TUEfrom, group->kcfg_TUEto );
    group->kcfg_TUEgraphics->setScene(my_scene);
    group->kcfg_TUEgraphics->setMouseTracking(TRUE);
    group->kcfg_TUEgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_TUEgraphics->setInteractive(TRUE);
    group->kcfg_TUEgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_TUEgraphics->show();
    group->kcfg_TUEfrom->hide();
    group->kcfg_TUEto->hide();

    group->kcfg_WEDruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_WEDruler->setMediumMarkDistance(8);
    group->kcfg_WEDruler->setOffset(0);
    my_scene = new MyScene(group->widget_WED, group->kcfg_WEDgraphics->geometry(), group->kcfg_WEDfrom, group->kcfg_WEDto );
    group->kcfg_WEDgraphics->setScene(my_scene);
    group->kcfg_WEDgraphics->setMouseTracking(TRUE);
    group->kcfg_WEDgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_WEDgraphics->setInteractive(TRUE);
    group->kcfg_WEDgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_WEDgraphics->show();
    group->kcfg_WEDfrom->hide();
    group->kcfg_WEDto->hide();

    group->kcfg_THUruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_THUruler->setMediumMarkDistance(8);
    group->kcfg_THUruler->setOffset(0);
    my_scene = new MyScene(group->widget_THU, group->kcfg_THUgraphics->geometry(), group->kcfg_THUfrom, group->kcfg_THUto );
    group->kcfg_THUgraphics->setScene(my_scene);
    group->kcfg_THUgraphics->setMouseTracking(TRUE);
    group->kcfg_THUgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_THUgraphics->setInteractive(TRUE);
    group->kcfg_THUgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_THUgraphics->show();
    group->kcfg_THUfrom->hide();
    group->kcfg_THUto->hide();

    group->kcfg_FRIruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_FRIruler->setMediumMarkDistance(8);
    group->kcfg_FRIruler->setOffset(0);
    my_scene = new MyScene(group->widget_FRI, group->kcfg_FRIgraphics->geometry(), group->kcfg_FRIfrom, group->kcfg_FRIto );
    group->kcfg_FRIgraphics->setScene(my_scene);
    group->kcfg_FRIgraphics->setMouseTracking(TRUE);
    group->kcfg_FRIgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_FRIgraphics->setInteractive(TRUE);
    group->kcfg_FRIgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_FRIgraphics->show();
    group->kcfg_FRIfrom->hide();
    group->kcfg_FRIto->hide();

    group->kcfg_SATruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_SATruler->setMediumMarkDistance(8);
    group->kcfg_SATruler->setOffset(0);
    my_scene = new MyScene(group->widget_SAT, group->kcfg_SATgraphics->geometry(), group->kcfg_SATfrom, group->kcfg_SATto );
    group->kcfg_SATgraphics->setScene(my_scene);
    group->kcfg_SATgraphics->setMouseTracking(TRUE);
    group->kcfg_SATgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_SATgraphics->setInteractive(TRUE);
    group->kcfg_SATgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_SATgraphics->show();
    group->kcfg_SATfrom->hide();
    group->kcfg_SATto->hide();

    group->kcfg_SUNruler->setRulerMetricStyle(KRuler::Custom);
    group->kcfg_SUNruler->setMediumMarkDistance(8);
    group->kcfg_SUNruler->setOffset(0);
    my_scene = new MyScene(group->widget_SUN, group->kcfg_SUNgraphics->geometry(), group->kcfg_SUNfrom, group->kcfg_SUNto );
    group->kcfg_SUNgraphics->setScene(my_scene);
    group->kcfg_SUNgraphics->setMouseTracking(TRUE);
    group->kcfg_SUNgraphics->viewport()->setMouseTracking(TRUE);
    group->kcfg_SUNgraphics->setInteractive(TRUE);
    group->kcfg_SUNgraphics->setFocusPolicy(Qt::StrongFocus);
    group->kcfg_SUNgraphics->show();
    group->kcfg_SUNfrom->hide();
    group->kcfg_SUNto->hide();

}




#include "KchildlockKCM.moc"
