/***************************************************************************
 *   Copyright (C) 2010 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KCHILDLOCKKCM_H
#define KCHILDLOCKKCM_H

#include <kcmodule.h>
#include <QDBusConnection>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kconfig.h>
#include <kpagedialog.h>
#include <kconfiggroup.h>

#include "../common/kchildlockcommon.h"
#include "kchildlock_general.h"
#include "kchildlock_user.h"
#include "kchildlock_group.h"
#include "ui_generalPage.h"
#include "ui_userPage.h"
#include "ui_groupPage.h"
#include "kchildlock_scene.h"

#define USERIDLOWLIMIT 200
#define GROUPIDLOWLIMIT 100



class QVBoxLayout;
class QWidget;



class KchildlockKCM : public KCModule, public KConfig
{
    Q_OBJECT

public:
    KchildlockKCM(QWidget *parent, const QVariantList& = QVariantList());
    ~KchildlockKCM();
    void load();
    void save();
    QString quickHelp() const;

signals:
     void settingsChanged(const QString& mystr);
     void updateMe();
    
private:
    KPageDialog *dialog;
    KchildlockGeneralSettings *myGeneralSettings;    
    KchildlockUserSettings *myUser1Settings;    
    KchildlockUserSettings *myUser2Settings;    
    KchildlockUserSettings *myUser3Settings;    
    KchildlockUserSettings *myUser4Settings;    
    KchildlockUserSettings *myUser5Settings;    
    KchildlockGroupSettings *myGroup1Settings;    
    KchildlockGroupSettings *myGroup2Settings;    
    KchildlockGroupSettings *myGroup3Settings;    
    Ui_generalPage *general;
    Ui_userPage *user1;
    Ui_userPage *user2;
    Ui_userPage *user3;
    Ui_userPage *user4;
    Ui_userPage *user5;
    Ui_groupPage *group1;
    Ui_groupPage *group2;
    Ui_groupPage *group3;
    KPageWidgetItem *generalPageItem;
    KPageWidgetItem *user1PageItem;
    KPageWidgetItem *user2PageItem;
    KPageWidgetItem *user3PageItem;
    KPageWidgetItem *user4PageItem;
    KPageWidgetItem *user5PageItem;
    KPageWidgetItem *group1PageItem;
    KPageWidgetItem *group2PageItem;
    KPageWidgetItem *group3PageItem;
    
    MyScene *my_scene;

    QStringList getlistofusers();
    QStringList getlistofgroups();
    
    void connect_general(Ui_generalPage *general_ui);
    void connect_user(Ui_userPage *user_ui);
    void connect_group(Ui_groupPage *group_ui);
    void disableDifferentSettings_user(Ui_userPage *user_ui);
    void disableSameSettings_user(Ui_userPage *user_ui);
    void disableDifferentSettings_group(Ui_groupPage *group_ui);
    void disableSameSettings_group(Ui_groupPage *group_ui);
    void fillValuesInGui_general();
    void fillValuesInGui_user(Ui_userPage *user, KchildlockUserSettings *mySettings);
    void fillValuesInGui_group(Ui_groupPage *group, KchildlockGroupSettings *mySettings);
    void setupUi_part2(Ui_userPage *user, KchildlockUserSettings *mySettings, QString numberstring);
    void setupUi_part3(Ui_groupPage *group, KchildlockGroupSettings *mySettings, QString numberstring);

    void setupUi_MON_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_TUE_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_WED_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_THU_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_FRI_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_SAT_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_SUN_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_DAY_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_WEEK_user(bool disabled, int visible, Ui_userPage *user_ui);

    void setupUi_APP1_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_APP2_user(bool disabled, int visible, Ui_userPage *user_ui);
    void setupUi_APP3_user(bool disabled, int visible, Ui_userPage *user_ui);

    void setupUi_MON_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_TUE_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_WED_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_THU_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_FRI_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_SAT_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_SUN_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_DAY_group(bool disabled, int visible, Ui_groupPage *group_ui);
    void setupUi_WEEK_group(bool disabled, int visible, Ui_groupPage *group_ui);
    
    Ui_userPage *current_userPage();
    Ui_groupPage *current_groupPage();
    KchildlockUserSettings *mycurrent_UserSettings;    
    KchildlockGroupSettings *mycurrent_GroupSettings;    

private slots:

//  Slots for General Settings
    void emitChanged_enable_restrictions(int);
    void emitChanged_enable_systray(int);
    void emitChanged_scaninterval(int);

//  Slots for User1...5
    void emitChanged_enabled_user(int);
    void emitChanged_MONenabled_user(int);
    void emitChanged_TUEenabled_user(int);
    void emitChanged_WEDenabled_user(int);
    void emitChanged_THUenabled_user(int);
    void emitChanged_FRIenabled_user(int);
    void emitChanged_SATenabled_user(int);
    void emitChanged_SUNenabled_user(int);
    void emitChanged_WEEKenabled_user(int);
    void emitChanged_username_user(QString);
    void emitChanged_sameRestrict_user(bool);
    void emitChanged_diffRestrict_user(bool);
    void emitChanged_value_user(int);
    void emitChanged_appname_user(QString);
    void emitChanged_APPenabled_user(int);

//  Slots for Group1...3
    void emitChanged_enabled_group(int);
    void emitChanged_MONenabled_group(int);
    void emitChanged_TUEenabled_group(int);
    void emitChanged_WEDenabled_group(int);
    void emitChanged_THUenabled_group(int);
    void emitChanged_FRIenabled_group(int);
    void emitChanged_SATenabled_group(int);
    void emitChanged_SUNenabled_group(int);
    void emitChanged_WEEKenabled_group(int);
    void emitChanged_groupname_group(QString);
    void emitChanged_sameRestrict_group(bool);
    void emitChanged_diffRestrict_group(bool);
    void emitChanged_value_group(int);

};


#endif /*KCHILDLOCKKCM_H*/
